"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var _a, _b;
Object.defineProperty(exports, "__esModule", { value: true });
var Sequelize = require("sequelize");
var Op = Sequelize.Op;
var CONNECTING_OPERATORS;
(function (CONNECTING_OPERATORS) {
    CONNECTING_OPERATORS["OR"] = "OR";
    CONNECTING_OPERATORS["AND"] = "AND";
})(CONNECTING_OPERATORS = exports.CONNECTING_OPERATORS || (exports.CONNECTING_OPERATORS = {}));
var OPERATORS;
(function (OPERATORS) {
    OPERATORS["EQ"] = "EQ";
    OPERATORS["NOT"] = "NOT";
    OPERATORS["IN"] = "IN";
    OPERATORS["NOTIN"] = "NOTIN";
    OPERATORS["BETWEEN"] = "BETWEEN";
    OPERATORS["NOTBETWEEN"] = "NOTBETWEEN";
    OPERATORS["LT"] = "LT";
    OPERATORS["LTE"] = "LTE";
    OPERATORS["GT"] = "GT";
    OPERATORS["GTE"] = "GTE";
    OPERATORS["NULL"] = "NULL";
    OPERATORS["LIKE"] = "LIKE";
    OPERATORS["NOTLIKE"] = "NOTLIKE";
})(OPERATORS = exports.OPERATORS || (exports.OPERATORS = {}));
var SEQUELIZE_OP_MAP = (_a = {},
    _a[CONNECTING_OPERATORS.OR] = Op.or,
    _a[CONNECTING_OPERATORS.AND] = Op.and,
    _a[OPERATORS.EQ] = Op.eq,
    _a[OPERATORS.NOT] = Op.not,
    _a[OPERATORS.IN] = Op.in,
    _a[OPERATORS.NOTIN] = Op.notIn,
    _a[OPERATORS.BETWEEN] = Op.between,
    _a[OPERATORS.NOTBETWEEN] = Op.notBetween,
    _a[OPERATORS.LT] = Op.lt,
    _a[OPERATORS.LTE] = Op.lte,
    _a[OPERATORS.GT] = Op.gt,
    _a[OPERATORS.GTE] = Op.gte,
    _a[OPERATORS.NULL] = Op.is,
    _a[OPERATORS.LIKE] = Op.like,
    _a[OPERATORS.NOTLIKE] = Op.notLike,
    _a);
var OPERTAOR_VALUE_TRANSFORMER = (_b = {},
    _b[OPERATORS.LIKE] = function (val) { return "%" + val + "%"; },
    _b[OPERATORS.NOTLIKE] = function (val) { return "%" + val + "%"; },
    _b[OPERATORS.IN] = function (val) { return val.split("-."); },
    _b[OPERATORS.NOTIN] = function (val) { return val.split("-."); },
    _b[OPERATORS.BETWEEN] = function (val) {
        return Array.isArray(val) ? val : val.split(",");
    },
    _b);
module.exports.SequelizeQueryStringParser = function (req, res, next) {
    var _a = req.query, filter = _a.filter, order = _a.order, _b = _a.page, queryPage = _b === void 0 ? 1 : _b, perPage = _a.perPage, includes_filter = _a.includes_filter, includes = _a.includes;
    var page = queryPage > 0 ? queryPage : 1;
    /**
     * Parse query string into sequelize readable objects
     */
    var sFilter = filter ? parseFilter(filter) : null;
    var sOrder = order ? parseOrder(order) : null;
    var parsedIncludes = includes
        ? includes
            .split(",")
            .map(function (association) { return parseIncludes(association); })
        : null;
    if (includes_filter) {
        Object.keys(includes_filter).forEach(function (key) {
            parsedIncludes = parsedIncludes.map(function (association) {
                if (key === association.association) {
                    return __assign(__assign({}, association), { where: parseFilter(includes_filter[key]) });
                }
                else {
                    return association;
                }
            });
        });
    }
    /**
     * Set the sequelize query object to the request so that it can be used later
     */
    req.sequelizeOptions = {
        distinct: parsedIncludes && parsedIncludes.length > 0,
        include: parsedIncludes,
        limit: perPage ? perPage : null,
        offset: perPage && page ? perPage * (page - 1) : 0,
        order: [sOrder],
        where: sFilter
    };
    next();
};
var parseFilter = function (filterObject) {
    var _a, _b, _c;
    var _d = filterObject.op, op = _d === void 0 ? null : _d, _e = filterObject.key, key = _e === void 0 ? null : _e, _f = filterObject.value, value = _f === void 0 ? null : _f;
    if (op === null) {
        return null;
    }
    if (Object.values(CONNECTING_OPERATORS).includes(op)) {
        return _a = {},
            _a[SEQUELIZE_OP_MAP[op]] = value.map(function (val) { return (__assign({}, parseFilter(val))); }),
            _a;
    }
    if (Object.values(OPERATORS).includes(op) && key !== null) {
        return _b = {},
            _b[key] = (_c = {},
                _c[SEQUELIZE_OP_MAP[op]] = OPERTAOR_VALUE_TRANSFORMER[op]
                    ? OPERTAOR_VALUE_TRANSFORMER[op](value)
                    : value,
                _c),
            _b;
    }
    return null;
};
var parseIncludes = function (includesText) {
    var associationNames = includesText.split(/\.(.+)/);
    return __assign(__assign({ association: associationNames[0].replace("*", "") }, (associationNames.length > 1
        ? { include: parseIncludes(associationNames[1]) }
        : {})), { required: associationNames[0].includes("*") });
};
var parseOrder = function (orderString) {
    return orderString.split(" ");
};
module.exports.parseFilter = parseFilter;
module.exports.parseOrder = parseOrder;
